function C=capcalc(makeplots,whitedBm,nextcanc,echocanc,fextcanc,P,CAT5E);
% 
% Function computes capacity based on agreed cable models as of May 6,2003 ad hoc
% conference call.  Alien crosstalk will be added later
%
% Function made available for IEEE802.3 10GBase-T SG
% written by Carlos Aldana, Solarflare Communications may 9, 2003
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The frequency range used to estimate the channel capacity is [0,intupperlimit]
% C is the capacity
% makeplots = 1 to generate plots
% whitedBm background noise level in dBm/Hz
% nextcanc - NEXT cancellation level (dB)
% echocanc - echo cancellation level (dB)
% fextcanc - FEXT cancellation level (dB)
% P input power (dBm)
% CAT5E = 1 for cat5 models, otherwise cat6
% The frequency range used to estimate the channel capacity is [0,intupperlimit]
% intupperlimit (in MHz)
%
if nargin==0,
    makeplots=1;
    whitedBm=-155;
    nextcanc=40;
    echocanc=50;
    fextcanc=20;
    P=10;
    CAT5E=1;
end
    if CAT5E==0,
        df=750e3; %this is for may 6 data
        intupperlimit=649.75;
    else
        intupperlimit=500.69;
        df=812.5e3;
    end
intupperlimit=intupperlimit*1e6;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Insertion Loss 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if CAT5E==0,
    load may6ilfreqcat6.txt
    freq=may6ilfreqcat6(:,1);
    IL1=-may6ilfreqcat6(:,2);
else
    load april17ilfreqcat5.txt 
    freq=april17ilfreqcat5(:,1);
    IL1=-april17ilfreqcat5(:,2);
end
if makeplots
    figure(1)
    plot(freq,IL1);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NEXT 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if CAT5E==0,
    load may6nxcat6.txt
    NX1=10*log10(10.^(-may6nxcat6(:,1)/10)+10.^(-may6nxcat6(:,2)/10)+10.^(-may6nxcat6(:,3)/10));
else
    load april17nxcat5.txt
    NX1=10*log10(10.^(-april17nxcat5(:,1)/10)+10.^(-april17nxcat5(:,2)/10)+10.^(-april17nxcat5(:,3)/10));
end        
if makeplots,
    figure(2)
    hold on
    plot(freq,NX1)
    grid
    title('next')
end
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FEXT 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if CAT5E==0,
    load may6fxcat6.txt
    FX1=10*log10(10.^(-may6fxcat6(:,1)/10)+10.^(-may6fxcat6(:,2)/10)+10.^(-may6fxcat6(:,3)/10));
else
    load april17fxcat5.txt
    FX1=10*log10(10.^(-april17fxcat5(:,1)/10)+10.^(-april17fxcat5(:,2)/10)+10.^(-april17fxcat5(:,3)/10));
end        
if makeplots,
    figure(3)
    hold on
    plot(freq,FX1)
    grid
    title('fext')
end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Return Loss 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if CAT5E==0,
    load may6rlcat6.txt
    RL1=-may6rlcat6;
else
    load april17rlcat5.txt 
    RL1=-april17rlcat5;
end
if makeplots,
    figure(4)
    hold on
    plot([freq],[RL1],'g');
    title('return loss')
    grid
end
         
   


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input signal PSD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Si=10*log10(10^(P/10)/intupperlimit)*ones(length(freq),1); % dBm/Hz %make it a column.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% signal and noise PSD @ receiver side
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    So1=Si+IL1;So2=Si+IL1;So3=Si+IL1;So4=Si+IL1;
    NEXT1=Si+[NX1];NEXT2=Si+[NX1];NEXT3=Si+[NX1];NEXT4=Si+[NX1];
 
    FEXT1=Si+[FX1];FEXT2=Si+[FX1];FEXT3=Si+[FX1];FEXT4=Si+[FX1];
    ECHO1=Si+RL1;ECHO2=Si+RL1;ECHO3=Si+RL1;ECHO4=Si+RL1;
    So=[So1 So2 So3 So4];
    BKN=whitedBm*ones(length(Si),4);
    NEXT=[NEXT1 NEXT2 NEXT3 NEXT4];
    FEXT=[FEXT1 FEXT2 FEXT3 FEXT4];
    ECHO=[ECHO1 ECHO2 ECHO3 ECHO4];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SNR after DSP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
        rBKN=BKN;
        rNEXT=NEXT-nextcanc;
        rECHO=ECHO-echocanc;
        rFEXT=FEXT-fextcanc;
      
  
    N=10.^(rBKN/10)+10.^(rNEXT/10)+10.^(rECHO/10)+10.^(rFEXT/10);
   

    S=10.^(So/10);
   
    SNR = S./N;
    C_f=log2(1+SNR); 
    C=sum(sum(C_f))*df ;
    
if makeplots
    figure(5); plot(freq,Si,':',freq,So(:,1),'--',freq,NEXT(:,1),'-.',freq,FEXT(:,1),'-x',freq,ECHO(:,1),'-+',freq,BKN(:,1),'-*')
    grid
    legend('signal in','signal out', 'next','fext','echo','awgn',-1)

    figure(6); plot(freq,10*log10(S(:,1)),'--',freq,rNEXT(:,1),'-.',freq,rFEXT(:,1),'-x',freq,rECHO(:,1),'-+',freq,rBKN(:,1),'-*') %,freq,10*log10(N),'-')
    grid
    legend('signal out','rem next','rem fext','rem echo','rem awgn',-1)
end
